@@ -1,10 +1,12 @@
 package main
 
 import (
+	"crypto/sha256"
 	"crypto/x509"
 	"encoding/pem"
 	"fmt"
 	"net"
+	"net/http"
 	"net/url"
 	"os"
 	"path/filepath"
@@ -15,13 +17,10 @@ import (
 
 	"golang.org/x/crypto/ssh/terminal"
 
-	"github.com/lxc/lxd/client"
-	"github.com/lxc/lxd/lxc/config"
+	"github.com/lxc/lxd"
 	"github.com/lxc/lxd/shared"
-	"github.com/lxc/lxd/shared/api"
 	"github.com/lxc/lxd/shared/gnuflag"
 	"github.com/lxc/lxd/shared/i18n"
-	"github.com/lxc/lxd/shared/logger"
 )
 
 type remoteCmd struct {
@@ -37,30 +36,16 @@ func (c *remoteCmd) showByDefault() bool {
 
 func (c *remoteCmd) usage() string {
 	return i18n.G(
-		`Usage: lxc remote <subcommand> [options]
+		`Manage remote LXD servers.
 
-Manage the list of remote LXD servers.
-
-lxc remote add <remote> <IP|FQDN|URL> [--accept-certificate] [--password=PASSWORD] [--public] [--protocol=PROTOCOL]
-    Add the remote <remote> at <url>.
-
-lxc remote remove <remote>
-    Remove the remote <remote>.
-
-lxc remote list
-    List all remotes.
-
-lxc remote rename <old name> <new name>
-    Rename remote <old name> to <new name>.
-
-lxc remote set-url <remote> <url>
-    Update <remote>'s url to <url>.
-
-lxc remote set-default <remote>
-    Set the default remote.
-
-lxc remote get-default
-    Print the default remote.`)
+lxc remote add <name> <url> [--accept-certificate] [--password=PASSWORD]
+                            [--public] [--protocol=PROTOCOL]                Add the remote <name> at <url>.
+lxc remote remove <name>                                                    Remove the remote <name>.
+lxc remote list                                                             List all remotes.
+lxc remote rename <old> <new>                                               Rename remote <old> to <new>.
+lxc remote set-url <name> <url>                                             Update <name>'s url to <url>.
+lxc remote set-default <name>                                               Set the default remote.
+lxc remote get-default                                                      Print the default remote.`)
 }
 
 func (c *remoteCmd) flags() {
@@ -70,20 +55,64 @@ func (c *remoteCmd) flags() {
 	gnuflag.BoolVar(&c.public, "public", false, i18n.G("Public image server"))
 }
 
-func (c *remoteCmd) addServer(conf *config.Config, server string, addr string, acceptCert bool, password string, public bool, protocol string) error {
+func generateClientCertificate(config *lxd.Config) error {
+	// Generate a client certificate if necessary.  The default repositories are
+	// either local or public, neither of which requires a client certificate.
+	// Generation of the cert is delayed to avoid unnecessary overhead, e.g in
+	// testing scenarios where only the default repositories are used.
+	certf := config.ConfigPath("client.crt")
+	keyf := config.ConfigPath("client.key")
+	if !shared.PathExists(certf) || !shared.PathExists(keyf) {
+		fmt.Fprintf(os.Stderr, i18n.G("Generating a client certificate. This may take a minute...")+"\n")
+
+		return shared.FindOrGenCert(certf, keyf, true)
+	}
+	return nil
+}
+
+func getRemoteCertificate(address string) (*x509.Certificate, error) {
+	// Setup a permissive TLS config
+	tlsConfig, err := shared.GetTLSConfig("", "", "", nil)
+	if err != nil {
+		return nil, err
+	}
+
+	tlsConfig.InsecureSkipVerify = true
+	tr := &http.Transport{
+		TLSClientConfig: tlsConfig,
+		Dial:            shared.RFC3493Dialer,
+		Proxy:           shared.ProxyFromEnvironment,
+	}
+
+	// Connect
+	client := &http.Client{Transport: tr}
+	resp, err := client.Get(address)
+	if err != nil {
+		return nil, err
+	}
+
+	// Retrieve the certificate
+	if resp.TLS == nil || len(resp.TLS.PeerCertificates) == 0 {
+		return nil, fmt.Errorf(i18n.G("Unable to read remote TLS certificate"))
+	}
+
+	return resp.TLS.PeerCertificates[0], nil
+}
+
+func (c *remoteCmd) addServer(config *lxd.Config, server string, addr string, acceptCert bool, password string, public bool, protocol string) error {
 	var rScheme string
 	var rHost string
 	var rPort string
 
 	// Setup the remotes list
-	if conf.Remotes == nil {
-		conf.Remotes = make(map[string]config.Remote)
+	if config.Remotes == nil {
+		config.Remotes = make(map[string]lxd.RemoteConfig)
 	}
 
 	/* Complex remote URL parsing */
 	remoteURL, err := url.Parse(addr)
 	if err != nil {
-		remoteURL = &url.URL{Host: addr}
+		return err
 	}
 
 	// Fast track simplestreams
@@ -92,7 +121,7 @@ func (c *remoteCmd) addServer(conf *config.Config, server string, addr string, a
 			return fmt.Errorf(i18n.G("Only https URLs are supported for simplestreams"))
 		}
 
-		conf.Remotes[server] = config.Remote{Addr: addr, Public: true, Protocol: protocol}
+		config.Remotes[server] = lxd.RemoteConfig{Addr: addr, Public: true, Protocol: protocol}
 		return nil
 	}
 
@@ -147,38 +176,46 @@ func (c *remoteCmd) addServer(conf *config.Config, server string, addr string, a
 		addr = rScheme + "://" + rHost
 	}
 
-	/* Actually add the remote */
-	conf.Remotes[server] = config.Remote{Addr: addr, Protocol: protocol}
-
-	// Attempt to connect
-	var d interface{}
-	if public {
-		d, err = conf.GetImageServer(server)
-	} else {
-		d, err = conf.GetContainerServer(server)
+	// Finally, actually add the remote, almost...  If the remote is a private
+	// HTTPS server then we need to ensure we have a client certificate before
+	// adding the remote server.
+	if rScheme != "unix" && !public {
+		err = generateClientCertificate(config)
+		if err != nil {
+			return err
+		}
 	}
+	config.Remotes[server] = lxd.RemoteConfig{Addr: addr, Protocol: protocol}
 
-	// Handle Unix socket connections
-	if strings.HasPrefix(addr, "unix:") {
+	remote := config.ParseRemote(server)
+	d, err := lxd.NewClient(config, remote)
+	if err != nil {
 		return err
 	}
 
-	// Check if the system CA worked for the TLS connection
+	if strings.HasPrefix(addr, "unix:") {
+		// NewClient succeeded so there was a lxd there (we fingered
+		// it) so just accept it
+		return nil
+	}
+
 	var certificate *x509.Certificate
+
+	/* Attempt to connect using the system root CA */
+	_, err = d.GetServerConfig()
 	if err != nil {
 		// Failed to connect using the system CA, so retrieve the remote certificate
-		certificate, err = shared.GetRemoteCertificate(addr)
+		certificate, err = getRemoteCertificate(addr)
 		if err != nil {
 			return err
 		}
 	}
 
-	// Handle certificate prompt
 	if certificate != nil {
 		if !acceptCert {
-			digest := shared.CertFingerprint(certificate)
+			digest := sha256.Sum256(certificate.Raw)
 
-			fmt.Printf(i18n.G("Certificate fingerprint: %s")+"\n", digest)
+			fmt.Printf(i18n.G("Certificate fingerprint: %x")+"\n", digest)
 			fmt.Printf(i18n.G("ok (y/n)?") + " ")
 			line, err := shared.ReadStdin()
 			if err != nil {
@@ -190,13 +227,13 @@ func (c *remoteCmd) addServer(conf *config.Config, server string, addr string, a
 			}
 		}
 
-		dnam := conf.ConfigPath("servercerts")
+		dnam := d.Config.ConfigPath("servercerts")
 		err := os.MkdirAll(dnam, 0750)
 		if err != nil {
 			return fmt.Errorf(i18n.G("Could not create server cert dir"))
 		}
 
-		certf := fmt.Sprintf("%s/%s.crt", dnam, server)
+		certf := fmt.Sprintf("%s/%s.crt", dnam, d.Name)
 		certOut, err := os.Create(certf)
 		if err != nil {
 			return err
@@ -206,41 +243,27 @@ func (c *remoteCmd) addServer(conf *config.Config, server string, addr string, a
 		certOut.Close()
 
 		// Setup a new connection, this time with the remote certificate
-		if public {
-			d, err = conf.GetImageServer(server)
-		} else {
-			d, err = conf.GetContainerServer(server)
-		}
-
+		d, err = lxd.NewClient(config, remote)
 		if err != nil {
 			return err
 		}
 	}
 
-	// Handle public remotes
-	if public {
-		conf.Remotes[server] = config.Remote{Addr: addr, Public: true}
+	if d.IsPublic() || public {
+		config.Remotes[server] = lxd.RemoteConfig{Addr: addr, Public: true}
+
+		if _, err := d.GetServerConfig(); err != nil {
+			return err
+		}
+
 		return nil
 	}
 
-	// Get server information
-	srv, _, err := d.(lxd.ContainerServer).GetServer()
-	if err != nil {
-		return err
-	}
-
-	// Detect public remotes
-	if srv.Public {
-		conf.Remotes[server] = config.Remote{Addr: addr, Public: true}
+	if d.AmTrusted() {
+		// server already has our cert, so we're done
 		return nil
 	}
 
-	// Check if our cert is already trusted
-	if srv.Auth == "trusted" {
-		return nil
-	}
-
-	// Prompt for trust password
 	if password == "" {
 		fmt.Printf(i18n.G("Admin password for %s: "), server)
 		pwd, err := terminal.ReadPassword(0)
@@ -256,24 +279,12 @@ func (c *remoteCmd) addServer(conf *config.Config, server string, addr string, a
 		password = string(pwd)
 	}
 
-	// Add client certificate to trust store
-	req := api.CertificatesPost{
-		Password: password,
-	}
-	req.Type = "client"
-
-	err = d.(lxd.ContainerServer).CreateCertificate(req)
+	err = d.AddMyCertToServer(password)
 	if err != nil {
 		return err
 	}
 
-	// And check if trusted now
-	srv, _, err = d.(lxd.ContainerServer).GetServer()
-	if err != nil {
-		return err
-	}
-
-	if srv.Auth != "trusted" {
+	if !d.AmTrusted() {
 		return fmt.Errorf(i18n.G("Server doesn't trust us after adding our cert"))
 	}
 
@@ -281,16 +292,16 @@ func (c *remoteCmd) addServer(conf *config.Config, server string, addr string, a
 	return nil
 }
 
-func (c *remoteCmd) removeCertificate(conf *config.Config, remote string) {
-	certf := conf.ServerCertPath(remote)
-	logger.Debugf("Trying to remove %s", certf)
+func (c *remoteCmd) removeCertificate(config *lxd.Config, remote string) {
+	certf := config.ServerCertPath(remote)
+	shared.LogDebugf("Trying to remove %s", certf)
 
 	os.Remove(certf)
 }
 
-func (c *remoteCmd) run(conf *config.Config, args []string) error {
+func (c *remoteCmd) run(config *lxd.Config, args []string) error {
 	if len(args) < 1 {
-		return errUsage
+		return errArgs
 	}
 
 	switch args[0] {
@@ -299,14 +310,14 @@ func (c *remoteCmd) run(conf *config.Config, args []string) error {
 			return errArgs
 		}
 
-		if rc, ok := conf.Remotes[args[1]]; ok {
+		if rc, ok := config.Remotes[args[1]]; ok {
 			return fmt.Errorf(i18n.G("remote %s exists as <%s>"), args[1], rc.Addr)
 		}
 
-		err := c.addServer(conf, args[1], args[2], c.acceptCert, c.password, c.public, c.protocol)
+		err := c.addServer(config, args[1], args[2], c.acceptCert, c.password, c.public, c.protocol)
 		if err != nil {
-			delete(conf.Remotes, args[1])
-			c.removeCertificate(conf, args[1])
+			delete(config.Remotes, args[1])
+			c.removeCertificate(config, args[1])
 			return err
 		}
 
@@ -315,7 +326,7 @@ func (c *remoteCmd) run(conf *config.Config, args []string) error {
 			return errArgs
 		}
 
-		rc, ok := conf.Remotes[args[1]]
+		rc, ok := config.Remotes[args[1]]
 		if !ok {
 			return fmt.Errorf(i18n.G("remote %s doesn't exist"), args[1])
 		}
@@ -324,17 +335,17 @@ func (c *remoteCmd) run(conf *config.Config, args []string) error {
 			return fmt.Errorf(i18n.G("remote %s is static and cannot be modified"), args[1])
 		}
 
-		if conf.DefaultRemote == args[1] {
+		if config.DefaultRemote == args[1] {
 			return fmt.Errorf(i18n.G("can't remove the default remote"))
 		}
 
-		delete(conf.Remotes, args[1])
+		delete(config.Remotes, args[1])
 
-		c.removeCertificate(conf, args[1])
+		c.removeCertificate(config, args[1])
 
 	case "list":
 		data := [][]string{}
-		for name, rc := range conf.Remotes {
+		for name, rc := range config.Remotes {
 			strPublic := i18n.G("NO")
 			if rc.Public {
 				strPublic = i18n.G("YES")
@@ -350,7 +361,7 @@ func (c *remoteCmd) run(conf *config.Config, args []string) error {
 			}
 
 			strName := name
-			if name == conf.DefaultRemote {
+			if name == config.DefaultRemote {
 				strName = fmt.Sprintf("%s (%s)", name, i18n.G("default"))
 			}
 			data = append(data, []string{strName, rc.Addr, rc.Protocol, strPublic, strStatic})
@@ -377,7 +388,7 @@ func (c *remoteCmd) run(conf *config.Config, args []string) error {
 			return errArgs
 		}
 
-		rc, ok := conf.Remotes[args[1]]
+		rc, ok := config.Remotes[args[1]]
 		if !ok {
 			return fmt.Errorf(i18n.G("remote %s doesn't exist"), args[1])
 		}
@@ -386,13 +397,13 @@ func (c *remoteCmd) run(conf *config.Config, args []string) error {
 			return fmt.Errorf(i18n.G("remote %s is static and cannot be modified"), args[1])
 		}
 
-		if _, ok := conf.Remotes[args[2]]; ok {
+		if _, ok := config.Remotes[args[2]]; ok {
 			return fmt.Errorf(i18n.G("remote %s already exists"), args[2])
 		}
 
 		// Rename the certificate file
-		oldPath := filepath.Join(conf.ConfigPath("servercerts"), fmt.Sprintf("%s.crt", args[1]))
-		newPath := filepath.Join(conf.ConfigPath("servercerts"), fmt.Sprintf("%s.crt", args[2]))
+		oldPath := filepath.Join(config.ConfigPath("servercerts"), fmt.Sprintf("%s.crt", args[1]))
+		newPath := filepath.Join(config.ConfigPath("servercerts"), fmt.Sprintf("%s.crt", args[2]))
 		if shared.PathExists(oldPath) {
 			err := os.Rename(oldPath, newPath)
 			if err != nil {
@@ -400,11 +411,11 @@ func (c *remoteCmd) run(conf *config.Config, args []string) error {
 			}
 		}
 
-		conf.Remotes[args[2]] = rc
-		delete(conf.Remotes, args[1])
+		config.Remotes[args[2]] = rc
+		delete(config.Remotes, args[1])
 
-		if conf.DefaultRemote == args[1] {
-			conf.DefaultRemote = args[2]
+		if config.DefaultRemote == args[1] {
+			config.DefaultRemote = args[2]
 		}
 
 	case "set-url":
@@ -412,7 +423,7 @@ func (c *remoteCmd) run(conf *config.Config, args []string) error {
 			return errArgs
 		}
 
-		rc, ok := conf.Remotes[args[1]]
+		rc, ok := config.Remotes[args[1]]
 		if !ok {
 			return fmt.Errorf(i18n.G("remote %s doesn't exist"), args[1])
 		}
@@ -421,28 +432,28 @@ func (c *remoteCmd) run(conf *config.Config, args []string) error {
 			return fmt.Errorf(i18n.G("remote %s is static and cannot be modified"), args[1])
 		}
 
-		conf.Remotes[args[1]] = config.Remote{Addr: args[2]}
+		config.Remotes[args[1]] = lxd.RemoteConfig{Addr: args[2]}
 
 	case "set-default":
 		if len(args) != 2 {
 			return errArgs
 		}
 
-		_, ok := conf.Remotes[args[1]]
+		_, ok := config.Remotes[args[1]]
 		if !ok {
 			return fmt.Errorf(i18n.G("remote %s doesn't exist"), args[1])
 		}
-		conf.DefaultRemote = args[1]
+		config.DefaultRemote = args[1]
 
 	case "get-default":
 		if len(args) != 1 {
 			return errArgs
 		}
-		fmt.Println(conf.DefaultRemote)
+		fmt.Println(config.DefaultRemote)
 		return nil
 	default:
 		return errArgs
 	}
 
-	return conf.SaveConfig(configPath)
+	return lxd.SaveConfig(config, configPath)
 }
