@@ -3,18 +3,20 @@ package main
 import (
 	"fmt"
 	"os"
+	"os/exec"
 	"path/filepath"
 	"strings"
 
 	"github.com/gorilla/websocket"
 
 	"github.com/lxc/lxd/shared"
-	"github.com/lxc/lxd/shared/idmap"
 
 	log "gopkg.in/inconshreveable/log15.v2"
 )
 
 type storageDir struct {
+	d *Daemon
+
 	storageShared
 }
 
@@ -34,15 +36,10 @@ func (s *storageDir) ContainerCreate(container container) error {
 		return fmt.Errorf("Error creating containers directory")
 	}
 
-	var mode os.FileMode
 	if container.IsPrivileged() {
-		mode = 0700
-	} else {
-		mode = 0755
-	}
-
-	if err := os.Chmod(cPath, mode); err != nil {
-		return err
+		if err := os.Chmod(cPath, 0700); err != nil {
+			return err
+		}
 	}
 
 	return container.TemplateApply("create")
@@ -56,19 +53,14 @@ func (s *storageDir) ContainerCreateFromImage(
 		return fmt.Errorf("Error creating rootfs directory")
 	}
 
-	var mode os.FileMode
 	if container.IsPrivileged() {
-		mode = 0700
-	} else {
-		mode = 0755
-	}
-
-	if err := os.Chmod(container.Path(), mode); err != nil {
-		return err
+		if err := os.Chmod(container.Path(), 0700); err != nil {
+			return err
+		}
 	}
 
 	imagePath := shared.VarPath("images", imageFingerprint)
-	if err := unpackImage(imagePath, container.Path(), s.storage.GetStorageType()); err != nil {
+	if err := unpackImage(imagePath, container.Path()); err != nil {
 		s.ContainerDelete(container)
 		return err
 	}
@@ -97,7 +89,7 @@ func (s *storageDir) ContainerDelete(container container) error {
 	err := os.RemoveAll(cPath)
 	if err != nil {
 		// RemovaAll fails on very long paths, so attempt an rm -Rf
-		output, err := shared.RunCommand("rm", "-Rf", cPath)
+		output, err := exec.Command("rm", "-Rf", cPath).CombinedOutput()
 		if err != nil {
 			s.log.Error("ContainerDelete: failed", log.Ctx{"cPath": cPath, "output": output})
 			return fmt.Errorf("Error cleaning up %s: %s", cPath, string(output))
@@ -131,11 +123,11 @@ func (s *storageDir) ContainerCopy(
 	return container.TemplateApply("copy")
 }
 
-func (s *storageDir) ContainerStart(name string, path string) error {
+func (s *storageDir) ContainerStart(container container) error {
 	return nil
 }
 
-func (s *storageDir) ContainerStop(name string, path string) error {
+func (s *storageDir) ContainerStop(container container) error {
 	return nil
 }
 
@@ -203,36 +195,13 @@ func (s *storageDir) ContainerSnapshotCreate(
 	/*
 	 * Copy by using rsync
 	 */
-	rsync := func(snapshotContainer container, oldPath string, newPath string) error {
-		output, err := storageRsyncCopy(oldPath, newPath)
-		if err != nil {
-			s.ContainerDelete(snapshotContainer)
-			s.log.Error("ContainerSnapshotCreate: rsync failed",
-				log.Ctx{"output": string(output)})
+	output, err := storageRsyncCopy(oldPath, newPath)
+	if err != nil {
+		s.ContainerDelete(snapshotContainer)
+		s.log.Error("ContainerSnapshotCreate: rsync failed",
+			log.Ctx{"output": string(output)})
 
-			return fmt.Errorf("rsync failed: %s", string(output))
-		}
-		return nil
-	}
-
-	if err := rsync(snapshotContainer, oldPath, newPath); err != nil {
-		return err
-	}
-
-	if sourceContainer.IsRunning() {
-		/* This is done to ensure consistency when snapshotting. But we
-		 * probably shouldn't fail just because of that.
-		 */
-		s.log.Debug("ContainerSnapshotCreate: trying to freeze and rsync again to ensure consistency.")
-		if err := sourceContainer.Freeze(); err != nil {
-			s.log.Warn("ContainerSnapshotCreate: trying to freeze and rsync again failed.")
-			return nil
-		}
-		defer sourceContainer.Unfreeze()
-
-		if err := rsync(snapshotContainer, oldPath, newPath); err != nil {
-			return err
-		}
+		return fmt.Errorf("rsync failed: %s", string(output))
 	}
 
 	return nil
@@ -313,6 +282,6 @@ func (s *storageDir) MigrationSource(container container) (MigrationStorageSourc
 	return rsyncMigrationSource(container)
 }
 
-func (s *storageDir) MigrationSink(live bool, container container, snapshots []*Snapshot, conn *websocket.Conn, srcIdmap *idmap.IdmapSet) error {
+func (s *storageDir) MigrationSink(live bool, container container, snapshots []*Snapshot, conn *websocket.Conn, srcIdmap *shared.IdmapSet) error {
 	return rsyncMigrationSink(live, container, snapshots, conn, srcIdmap)
 }
