@@ -1,7 +1,6 @@
 package main
 
 import (
-	"database/sql"
 	"fmt"
 	"io/ioutil"
 	"os"
@@ -13,18 +12,15 @@ import (
 
 	"github.com/gorilla/websocket"
 
-	"github.com/lxc/lxd/lxd/db"
 	"github.com/lxc/lxd/shared"
-	"github.com/lxc/lxd/shared/idmap"
-	"github.com/lxc/lxd/shared/logger"
 
 	log "gopkg.in/inconshreveable/log15.v2"
 )
 
 func storageLVMCheckVolumeGroup(vgName string) error {
-	output, err := shared.RunCommand("vgdisplay", "-s", vgName)
+	output, err := exec.Command("vgdisplay", "-s", vgName).CombinedOutput()
 	if err != nil {
-		logger.Debug("vgdisplay failed to find vg", log.Ctx{"output": string(output)})
+		shared.LogDebug("vgdisplay failed to find vg", log.Ctx{"output": string(output)})
 		return fmt.Errorf("LVM volume group '%s' not found", vgName)
 	}
 
@@ -52,14 +48,14 @@ func storageLVMThinpoolExists(vgName string, poolName string) (bool, error) {
 	return false, fmt.Errorf("Pool named '%s' exists but is not a thin pool.", poolName)
 }
 
-func storageLVMGetThinPoolUsers(dbObj *sql.DB) ([]string, error) {
+func storageLVMGetThinPoolUsers(d *Daemon) ([]string, error) {
 	results := []string{}
 
 	if daemonConfig["storage.lvm_vg_name"].Get() == "" {
 		return results, nil
 	}
 
-	cNames, err := db.ContainersList(dbObj, db.CTypeRegular)
+	cNames, err := dbContainersList(d.db, cTypeRegular)
 	if err != nil {
 		return results, err
 	}
@@ -77,7 +73,7 @@ func storageLVMGetThinPoolUsers(dbObj *sql.DB) ([]string, error) {
 		}
 	}
 
-	imageNames, err := db.ImagesGet(dbObj, false)
+	imageNames, err := dbImagesGet(d.db, false)
 	if err != nil {
 		return results, err
 	}
@@ -93,11 +89,7 @@ func storageLVMGetThinPoolUsers(dbObj *sql.DB) ([]string, error) {
 }
 
 func storageLVMValidateThinPoolName(d *Daemon, key string, value string) error {
-	return doStorageLVMValidateThinPoolName(d.db, key, value)
-}
-
-func doStorageLVMValidateThinPoolName(db *sql.DB, key string, value string) error {
-	users, err := storageLVMGetThinPoolUsers(db)
+	users, err := storageLVMGetThinPoolUsers(d)
 	if err != nil {
 		return fmt.Errorf("Error checking if a pool is already in use: %v", err)
 	}
@@ -126,7 +118,7 @@ func doStorageLVMValidateThinPoolName(db *sql.DB, key string, value string) erro
 }
 
 func storageLVMValidateVolumeGroupName(d *Daemon, key string, value string) error {
-	users, err := storageLVMGetThinPoolUsers(d.db)
+	users, err := storageLVMGetThinPoolUsers(d)
 	if err != nil {
 		return fmt.Errorf("Error checking if a pool is already in use: %v", err)
 	}
@@ -146,10 +138,10 @@ func storageLVMValidateVolumeGroupName(d *Daemon, key string, value string) erro
 }
 
 func xfsGenerateNewUUID(lvpath string) error {
-	output, err := shared.RunCommand(
+	output, err := exec.Command(
 		"xfs_admin",
 		"-U", "generate",
-		lvpath)
+		lvpath).CombinedOutput()
 	if err != nil {
 		return fmt.Errorf("Error generating new UUID: %v\noutput:'%s'", err, string(output))
 	}
@@ -163,6 +155,7 @@ func containerNameToLVName(containerName string) string {
 }
 
 type storageLvm struct {
+	d      *Daemon
 	vgName string
 
 	storageShared
@@ -175,7 +168,7 @@ func (s *storageLvm) Init(config map[string]interface{}) (storage, error) {
 		return s, err
 	}
 
-	output, err := shared.RunCommand("lvm", "version")
+	output, err := exec.Command("lvm", "version").CombinedOutput()
 	if err != nil {
 		return nil, fmt.Errorf("Error getting LVM version: %v\noutput:'%s'", err, string(output))
 	}
@@ -329,7 +322,8 @@ func (s *storageLvm) ContainerCreateFromImage(
 		}
 	}
 
-	err = tryMount(lvpath, destPath, fstype, 0, "discard")
+	mountOptions := daemonConfig["storage.lvm_mount_options"].Get()
+	err = tryMount(lvpath, destPath, fstype, 0, mountOptions)
 	if err != nil {
 		s.ContainerDelete(container)
 		return fmt.Errorf("Error mounting snapshot LV: %v", err)
@@ -410,7 +404,7 @@ func (s *storageLvm) ContainerCopy(container container, sourceContainer containe
 			return err
 		}
 
-		if err := s.ContainerStart(container.Name(), container.Path()); err != nil {
+		if err := s.ContainerStart(container); err != nil {
 			s.log.Error("Error starting/mounting container", log.Ctx{"err": err, "container": container.Name()})
 			s.ContainerDelete(container)
 			return err
@@ -425,35 +419,36 @@ func (s *storageLvm) ContainerCopy(container container, sourceContainer containe
 			return fmt.Errorf("rsync failed: %s", string(output))
 		}
 
-		if err := s.ContainerStop(container.Name(), container.Path()); err != nil {
+		if err := s.ContainerStop(container); err != nil {
 			return err
 		}
 	}
 	return container.TemplateApply("copy")
 }
 
-func (s *storageLvm) ContainerStart(name string, path string) error {
-	lvName := containerNameToLVName(name)
+func (s *storageLvm) ContainerStart(container container) error {
+	lvName := containerNameToLVName(container.Name())
 	lvpath := fmt.Sprintf("/dev/%s/%s", s.vgName, lvName)
 	fstype := daemonConfig["storage.lvm_fstype"].Get()
 
-	err := tryMount(lvpath, path, fstype, 0, "discard")
+	mountOptions := daemonConfig["storage.lvm_mount_options"].Get()
+	err := tryMount(lvpath, container.Path(), fstype, 0, mountOptions)
 	if err != nil {
 		return fmt.Errorf(
 			"Error mounting snapshot LV path='%s': %v",
-			path,
+			container.Path(),
 			err)
 	}
 
 	return nil
 }
 
-func (s *storageLvm) ContainerStop(name string, path string) error {
-	err := tryUnmount(path, 0)
+func (s *storageLvm) ContainerStop(container container) error {
+	err := tryUnmount(container.Path(), 0)
 	if err != nil {
 		return fmt.Errorf(
 			"failed to unmount container path '%s'.\nError: %v",
-			path,
+			container.Path(),
 			err)
 	}
 
@@ -564,7 +559,7 @@ func (s *storageLvm) createSnapshotContainer(
 
 	srcName := containerNameToLVName(sourceContainer.Name())
 	destName := containerNameToLVName(snapshotContainer.Name())
-	logger.Debug(
+	shared.LogDebug(
 		"Creating snapshot",
 		log.Ctx{"srcName": srcName, "destName": destName})
 
@@ -656,7 +651,7 @@ func (s *storageLvm) ContainerSnapshotStart(container container) error {
 	srcName := containerNameToLVName(container.Name())
 	destName := containerNameToLVName(container.Name() + "/rw")
 
-	logger.Debug(
+	shared.LogDebug(
 		"Creating snapshot",
 		log.Ctx{"srcName": srcName, "destName": destName})
 
@@ -682,7 +677,8 @@ func (s *storageLvm) ContainerSnapshotStart(container container) error {
 		}
 	}
 
-	err = tryMount(lvpath, container.Path(), fstype, 0, "discard")
+	mountOptions := daemonConfig["storage.lvm_mount_options"].Get()
+	err = tryMount(lvpath, container.Path(), fstype, 0, mountOptions)
 	if err != nil {
 		return fmt.Errorf(
 			"Error mounting snapshot LV path='%s': %v",
@@ -694,7 +690,7 @@ func (s *storageLvm) ContainerSnapshotStart(container container) error {
 }
 
 func (s *storageLvm) ContainerSnapshotStop(container container) error {
-	err := s.ContainerStop(container.Name(), container.Path())
+	err := s.ContainerStop(container)
 	if err != nil {
 		return err
 	}
@@ -737,13 +733,14 @@ func (s *storageLvm) ImageCreate(fingerprint string) error {
 	}()
 
 	fstype := daemonConfig["storage.lvm_fstype"].Get()
-	err = tryMount(lvpath, tempLVMountPoint, fstype, 0, "discard")
+	mountOptions := daemonConfig["storage.lvm_mount_options"].Get()
+	err = tryMount(lvpath, tempLVMountPoint, fstype, 0, mountOptions)
 	if err != nil {
-		logger.Infof("Error mounting image LV for unpacking: %v", err)
+		shared.LogInfof("Error mounting image LV for unpacking: %v", err)
 		return fmt.Errorf("Error mounting image LV: %v", err)
 	}
 
-	unpackErr := unpackImage(finalName, tempLVMountPoint, s.storage.GetStorageType())
+	unpackErr := unpackImage(finalName, tempLVMountPoint)
 
 	err = tryUnmount(tempLVMountPoint, 0)
 	if err != nil {
@@ -791,16 +788,16 @@ func (s *storageLvm) createDefaultThinPool() (string, error) {
 	}
 
 	// Create the thin pool
-	var output string
+	var output []byte
 	if isRecent {
-		output, err = shared.TryRunCommand(
+		output, err = tryExec(
 			"lvcreate",
 			"--poolmetadatasize", "1G",
 			"-l", "100%FREE",
 			"--thinpool",
 			fmt.Sprintf("%s/%s", s.vgName, thinPoolName))
 	} else {
-		output, err = shared.TryRunCommand(
+		output, err = tryExec(
 			"lvcreate",
 			"--poolmetadatasize", "1G",
 			"-L", "1G",
@@ -822,7 +819,7 @@ func (s *storageLvm) createDefaultThinPool() (string, error) {
 
 	if !isRecent {
 		// Grow it to the maximum VG size (two step process required by old LVM)
-		output, err = shared.TryRunCommand(
+		output, err = tryExec(
 			"lvextend",
 			"--alloc", "anywhere",
 			"-l", "100%FREE",
@@ -860,7 +857,7 @@ func (s *storageLvm) createThinLV(lvname string) (string, error) {
 			return "", fmt.Errorf("Error creating LVM thin pool: %v", err)
 		}
 
-		err = doStorageLVMValidateThinPoolName(s.s.DB, "", poolname)
+		err = storageLVMValidateThinPoolName(s.d, "", poolname)
 		if err != nil {
 			s.log.Error("Setting thin pool name", log.Ctx{"err": err})
 			return "", fmt.Errorf("Error setting LVM thin pool config: %v", err)
@@ -869,7 +866,7 @@ func (s *storageLvm) createThinLV(lvname string) (string, error) {
 
 	lvSize := daemonConfig["storage.lvm_volume_size"].Get()
 
-	output, err := shared.TryRunCommand(
+	output, err := tryExec(
 		"lvcreate",
 		"--thin",
 		"-n", lvname,
@@ -885,12 +882,12 @@ func (s *storageLvm) createThinLV(lvname string) (string, error) {
 	fstype := daemonConfig["storage.lvm_fstype"].Get()
 	switch fstype {
 	case "xfs":
-		output, err = shared.TryRunCommand(
+		output, err = tryExec(
 			"mkfs.xfs",
 			lvpath)
 	default:
 		// default = ext4
-		output, err = shared.TryRunCommand(
+		output, err = tryExec(
 			"mkfs.ext4",
 			"-E", "nodiscard,lazy_itable_init=0,lazy_journal_init=0",
 			lvpath)
@@ -906,9 +903,9 @@ func (s *storageLvm) createThinLV(lvname string) (string, error) {
 
 func (s *storageLvm) removeLV(lvname string) error {
 	var err error
-	var output string
+	var output []byte
 
-	output, err = shared.TryRunCommand(
+	output, err = tryExec(
 		"lvremove", "-f", fmt.Sprintf("%s/%s", s.vgName, lvname))
 
 	if err != nil {
@@ -925,16 +922,15 @@ func (s *storageLvm) createSnapshotLV(lvname string, origlvname string, readonly
 	if err != nil {
 		return "", fmt.Errorf("Error checking LVM version: %v", err)
 	}
-
-	var output string
+	var output []byte
 	if isRecent {
-		output, err = shared.TryRunCommand(
+		output, err = tryExec(
 			"lvcreate",
 			"-kn",
 			"-n", lvname,
 			"-s", fmt.Sprintf("/dev/%s/%s", s.vgName, origlvname))
 	} else {
-		output, err = shared.TryRunCommand(
+		output, err = tryExec(
 			"lvcreate",
 			"-n", lvname,
 			"-s", fmt.Sprintf("/dev/%s/%s", s.vgName, origlvname))
@@ -947,9 +943,9 @@ func (s *storageLvm) createSnapshotLV(lvname string, origlvname string, readonly
 	snapshotFullName := fmt.Sprintf("/dev/%s/%s", s.vgName, lvname)
 
 	if readonly {
-		output, err = shared.TryRunCommand("lvchange", "-ay", "-pr", snapshotFullName)
+		output, err = tryExec("lvchange", "-ay", "-pr", snapshotFullName)
 	} else {
-		output, err = shared.TryRunCommand("lvchange", "-ay", snapshotFullName)
+		output, err = tryExec("lvchange", "-ay", snapshotFullName)
 	}
 
 	if err != nil {
@@ -964,7 +960,7 @@ func (s *storageLvm) isLVMContainer(container container) bool {
 }
 
 func (s *storageLvm) renameLV(oldName string, newName string) (string, error) {
-	output, err := shared.TryRunCommand("lvrename", s.vgName, oldName, newName)
+	output, err := tryExec("lvrename", s.vgName, oldName, newName)
 	return string(output), err
 }
 
@@ -980,6 +976,6 @@ func (s *storageLvm) MigrationSource(container container) (MigrationStorageSourc
 	return rsyncMigrationSource(container)
 }
 
-func (s *storageLvm) MigrationSink(live bool, container container, snapshots []*Snapshot, conn *websocket.Conn, srcIdmap *idmap.IdmapSet) error {
+func (s *storageLvm) MigrationSink(live bool, container container, snapshots []*Snapshot, conn *websocket.Conn, srcIdmap *shared.IdmapSet) error {
 	return rsyncMigrationSink(live, container, snapshots, conn, srcIdmap)
 }
