@@ -8,20 +8,17 @@ import (
 	"strconv"
 	"strings"
 	"syscall"
-	"time"
 
 	"github.com/gorilla/websocket"
 
-	"github.com/lxc/lxd/lxd/util"
 	"github.com/lxc/lxd/shared"
-	"github.com/lxc/lxd/shared/idmap"
-	"github.com/lxc/lxd/shared/logger"
 
 	"github.com/pborman/uuid"
 	log "gopkg.in/inconshreveable/log15.v2"
 )
 
 type storageZfs struct {
+	d       *Daemon
 	zfsPool string
 
 	storageShared
@@ -55,10 +52,10 @@ func (s *storageZfs) Init(config map[string]interface{}) (storage, error) {
 	err = s.zfsCheckPool(s.zfsPool)
 	if err != nil {
 		if shared.PathExists(shared.VarPath("zfs.img")) {
-			_ = util.LoadModule("zfs")
+			_ = exec.Command("modprobe", "zfs").Run()
 
-			output, err := shared.RunCommand("zpool", "import",
-				"-d", shared.VarPath(), s.zfsPool)
+			output, err := exec.Command("zpool", "import",
+				"-d", shared.VarPath(), s.zfsPool).CombinedOutput()
 			if err != nil {
 				return s, fmt.Errorf("Unable to import the ZFS pool: %s", output)
 			}
@@ -67,7 +64,7 @@ func (s *storageZfs) Init(config map[string]interface{}) (storage, error) {
 		}
 	}
 
-	output, err := shared.RunCommand("zfs", "get", "version", "-H", "-o", "value", s.zfsPool)
+	output, err := exec.Command("zfs", "get", "version", "-H", "-o", "value", s.zfsPool).CombinedOutput()
 	if err != nil {
 		return s, fmt.Errorf("The 'zfs' tool isn't working properly")
 	}
@@ -77,13 +74,13 @@ func (s *storageZfs) Init(config map[string]interface{}) (storage, error) {
 		return s, fmt.Errorf("The 'zfs' tool isn't working properly")
 	}
 
-	output, err = shared.RunCommand("zfs", "get", "mountpoint", "-H", "-o", "source", s.zfsPool)
+	output, err = exec.Command("zfs", "get", "mountpoint", "-H", "-o", "source", s.zfsPool).CombinedOutput()
 	if err != nil {
 		return s, fmt.Errorf("Unable to query ZFS mountpoint")
 	}
 
 	if strings.TrimSpace(string(output)) != "local" {
-		_, err = shared.RunCommand("zfs", "set", "mountpoint=none", s.zfsPool)
+		err = shared.RunCommand("zfs", "set", "mountpoint=none", s.zfsPool)
 		if err != nil {
 			return s, err
 		}
@@ -93,8 +90,8 @@ func (s *storageZfs) Init(config map[string]interface{}) (storage, error) {
 }
 
 // Things we don't need to care about
-func (s *storageZfs) ContainerStart(name string, path string) error {
-	fs := fmt.Sprintf("containers/%s", name)
+func (s *storageZfs) ContainerStart(container container) error {
+	fs := fmt.Sprintf("containers/%s", container.Name())
 
 	// Just in case the container filesystem got unmounted
 	if !shared.IsMountPoint(shared.VarPath(fs)) {
@@ -104,7 +101,7 @@ func (s *storageZfs) ContainerStart(name string, path string) error {
 	return nil
 }
 
-func (s *storageZfs) ContainerStop(name string, path string) error {
+func (s *storageZfs) ContainerStop(container container) error {
 	return nil
 }
 
@@ -191,7 +188,11 @@ func (s *storageZfs) ContainerCanRestore(container container, sourceContainer co
 	}
 
 	if snaps[len(snaps)-1].Name() != sourceContainer.Name() {
-		return fmt.Errorf("ZFS can only restore from the latest snapshot. Delete newer snapshots or copy the snapshot into a new container instead.")
+		if !daemonConfig["storage.zfs_remove_snapshots"].GetBool() {
+			return fmt.Errorf("ZFS can only restore from the latest snapshot. Delete newer snapshots or copy the snapshot into a new container instead.")
+		}
+
+		return nil
 	}
 
 	return nil
@@ -394,11 +395,29 @@ func (s *storageZfs) ContainerRename(container container, newName string) error
 }
 
 func (s *storageZfs) ContainerRestore(container container, sourceContainer container) error {
+	// Remove any needed snapshot
+	snaps, err := container.Snapshots()
+	if err != nil {
+		return err
+	}
+
+	for i := len(snaps) - 1; i != 0; i-- {
+		if snaps[i].Name() == sourceContainer.Name() {
+			break
+		}
+
+		err := snaps[i].Delete()
+		if err != nil {
+			return err
+		}
+	}
+
+	// Restore the snapshot
 	fields := strings.SplitN(sourceContainer.Name(), shared.SnapshotDelimiter, 2)
 	cName := fields[0]
 	snapName := fmt.Sprintf("snapshot-%s", fields[1])
 
-	err := s.zfsSnapshotRestore(fmt.Sprintf("containers/%s", cName), snapName)
+	err = s.zfsSnapshotRestore(fmt.Sprintf("containers/%s", cName), snapName)
 	if err != nil {
 		return err
 	}
@@ -411,10 +430,15 @@ func (s *storageZfs) ContainerSetQuota(container container, size int64) error {
 
 	fs := fmt.Sprintf("containers/%s", container.Name())
 
+	property := "quota"
+	if daemonConfig["storage.zfs_use_refquota"].GetBool() {
+		property = "refquota"
+	}
+
 	if size > 0 {
-		err = s.zfsSet(fs, "quota", fmt.Sprintf("%d", size))
+		err = s.zfsSet(fs, property, fmt.Sprintf("%d", size))
 	} else {
-		err = s.zfsSet(fs, "quota", "none")
+		err = s.zfsSet(fs, property, "none")
 	}
 
 	if err != nil {
@@ -429,7 +453,12 @@ func (s *storageZfs) ContainerGetUsage(container container) (int64, error) {
 
 	fs := fmt.Sprintf("containers/%s", container.Name())
 
-	value, err := s.zfsGet(fs, "used")
+	property := "used"
+	if daemonConfig["storage.zfs_use_refquota"].GetBool() {
+		property = "usedbydataset"
+	}
+
+	value, err := s.zfsGet(fs, property)
 	if err != nil {
 		return -1, err
 	}
@@ -630,7 +659,7 @@ func (s *storageZfs) ImageCreate(fingerprint string) error {
 		return err
 	}
 
-	err = unpackImage(imagePath, subvol, s.storage.GetStorageType())
+	err = unpackImage(imagePath, subvol)
 	if err != nil {
 		return cleanup(err)
 	}
@@ -687,8 +716,8 @@ func (s *storageZfs) ImageDelete(fingerprint string) error {
 
 // Helper functions
 func (s *storageZfs) zfsCheckPool(pool string) error {
-	output, err := shared.RunCommand(
-		"zfs", "get", "type", "-H", "-o", "value", pool)
+	output, err := exec.Command(
+		"zfs", "get", "type", "-H", "-o", "value", pool).CombinedOutput()
 	if err != nil {
 		return fmt.Errorf(strings.Split(string(output), "\n")[0])
 	}
@@ -709,13 +738,13 @@ func (s *storageZfs) zfsClone(source string, name string, dest string, dotZfs bo
 		mountpoint += ".zfs"
 	}
 
-	output, err := shared.RunCommand(
+	output, err := exec.Command(
 		"zfs",
 		"clone",
 		"-p",
 		"-o", fmt.Sprintf("mountpoint=%s", mountpoint),
 		fmt.Sprintf("%s/%s@%s", s.zfsPool, source, name),
-		fmt.Sprintf("%s/%s", s.zfsPool, dest))
+		fmt.Sprintf("%s/%s", s.zfsPool, dest)).CombinedOutput()
 	if err != nil {
 		s.log.Error("zfs clone failed", log.Ctx{"output": string(output)})
 		return fmt.Errorf("Failed to clone the filesystem: %s", output)
@@ -742,13 +771,13 @@ func (s *storageZfs) zfsClone(source string, name string, dest string, dotZfs bo
 			mountpoint += ".zfs"
 		}
 
-		output, err := shared.RunCommand(
+		output, err := exec.Command(
 			"zfs",
 			"clone",
 			"-p",
 			"-o", fmt.Sprintf("mountpoint=%s", mountpoint),
 			fmt.Sprintf("%s/%s@%s", s.zfsPool, sub, name),
-			fmt.Sprintf("%s/%s", s.zfsPool, destSubvol))
+			fmt.Sprintf("%s/%s", s.zfsPool, destSubvol)).CombinedOutput()
 		if err != nil {
 			s.log.Error("zfs clone failed", log.Ctx{"output": string(output)})
 			return fmt.Errorf("Failed to clone the sub-volume: %s", output)
@@ -759,12 +788,12 @@ func (s *storageZfs) zfsClone(source string, name string, dest string, dotZfs bo
 }
 
 func (s *storageZfs) zfsCreate(path string) error {
-	output, err := shared.RunCommand(
+	output, err := exec.Command(
 		"zfs",
 		"create",
 		"-p",
 		"-o", fmt.Sprintf("mountpoint=%s.zfs", shared.VarPath(path)),
-		fmt.Sprintf("%s/%s", s.zfsPool, path))
+		fmt.Sprintf("%s/%s", s.zfsPool, path)).CombinedOutput()
 	if err != nil {
 		s.log.Error("zfs create failed", log.Ctx{"output": string(output)})
 		return fmt.Errorf("Failed to create ZFS filesystem: %s", output)
@@ -788,7 +817,7 @@ func (s *storageZfs) zfsDestroy(path string) error {
 	}
 
 	// Due to open fds or kernel refs, this may fail for a bit, give it 10s
-	output, err := shared.TryRunCommand(
+	output, err := tryExec(
 		"zfs",
 		"destroy",
 		"-r",
@@ -878,14 +907,14 @@ func (s *storageZfs) zfsExists(path string) bool {
 }
 
 func (s *storageZfs) zfsGet(path string, key string) (string, error) {
-	output, err := shared.RunCommand(
+	output, err := exec.Command(
 		"zfs",
 		"get",
 		"-H",
 		"-p",
 		"-o", "value",
 		key,
-		fmt.Sprintf("%s/%s", s.zfsPool, path))
+		fmt.Sprintf("%s/%s", s.zfsPool, path)).CombinedOutput()
 	if err != nil {
 		return string(output), fmt.Errorf("Failed to get ZFS config: %s", output)
 	}
@@ -894,41 +923,28 @@ func (s *storageZfs) zfsGet(path string, key string) (string, error) {
 }
 
 func (s *storageZfs) zfsRename(source string, dest string) error {
-	var err error
-	var output string
-
-	for i := 0; i < 20; i++ {
-		output, err = shared.RunCommand(
-			"zfs",
-			"rename",
-			"-p",
-			fmt.Sprintf("%s/%s", s.zfsPool, source),
-			fmt.Sprintf("%s/%s", s.zfsPool, dest))
-
-		// Success
-		if err == nil {
-			return nil
+	output, err := tryExec(
+		"zfs",
+		"rename",
+		"-p",
+		fmt.Sprintf("%s/%s", s.zfsPool, source),
+		fmt.Sprintf("%s/%s", s.zfsPool, dest))
+	if err != nil {
+		if s.zfsExists(source) || !s.zfsExists(dest) {
+			s.log.Error("zfs rename failed", log.Ctx{"output": string(output)})
+			return fmt.Errorf("Failed to rename ZFS filesystem: %s", output)
 		}
-
-		// zfs rename can fail because of descendants, yet still manage the rename
-		if !s.zfsExists(source) && s.zfsExists(dest) {
-			return nil
-		}
-
-		time.Sleep(500 * time.Millisecond)
 	}
 
-	// Timeout
-	s.log.Error("zfs rename failed", log.Ctx{"output": string(output)})
-	return fmt.Errorf("Failed to rename ZFS filesystem: %s", output)
+	return nil
 }
 
 func (s *storageZfs) zfsSet(path string, key string, value string) error {
-	output, err := shared.RunCommand(
+	output, err := exec.Command(
 		"zfs",
 		"set",
 		fmt.Sprintf("%s=%s", key, value),
-		fmt.Sprintf("%s/%s", s.zfsPool, path))
+		fmt.Sprintf("%s/%s", s.zfsPool, path)).CombinedOutput()
 	if err != nil {
 		s.log.Error("zfs set failed", log.Ctx{"output": string(output)})
 		return fmt.Errorf("Failed to set ZFS config: %s", output)
@@ -938,11 +954,11 @@ func (s *storageZfs) zfsSet(path string, key string, value string) error {
 }
 
 func (s *storageZfs) zfsSnapshotCreate(path string, name string) error {
-	output, err := shared.RunCommand(
+	output, err := exec.Command(
 		"zfs",
 		"snapshot",
 		"-r",
-		fmt.Sprintf("%s/%s@%s", s.zfsPool, path, name))
+		fmt.Sprintf("%s/%s@%s", s.zfsPool, path, name)).CombinedOutput()
 	if err != nil {
 		s.log.Error("zfs snapshot failed", log.Ctx{"output": string(output)})
 		return fmt.Errorf("Failed to create ZFS snapshot: %s", output)
@@ -952,11 +968,11 @@ func (s *storageZfs) zfsSnapshotCreate(path string, name string) error {
 }
 
 func (s *storageZfs) zfsSnapshotDestroy(path string, name string) error {
-	output, err := shared.RunCommand(
+	output, err := exec.Command(
 		"zfs",
 		"destroy",
 		"-r",
-		fmt.Sprintf("%s/%s@%s", s.zfsPool, path, name))
+		fmt.Sprintf("%s/%s@%s", s.zfsPool, path, name)).CombinedOutput()
 	if err != nil {
 		s.log.Error("zfs destroy failed", log.Ctx{"output": string(output)})
 		return fmt.Errorf("Failed to destroy ZFS snapshot: %s", output)
@@ -966,7 +982,7 @@ func (s *storageZfs) zfsSnapshotDestroy(path string, name string) error {
 }
 
 func (s *storageZfs) zfsSnapshotRestore(path string, name string) error {
-	output, err := shared.TryRunCommand(
+	output, err := tryExec(
 		"zfs",
 		"rollback",
 		fmt.Sprintf("%s/%s@%s", s.zfsPool, path, name))
@@ -990,7 +1006,7 @@ func (s *storageZfs) zfsSnapshotRestore(path string, name string) error {
 			continue
 		}
 
-		output, err := shared.TryRunCommand(
+		output, err := tryExec(
 			"zfs",
 			"rollback",
 			fmt.Sprintf("%s/%s@%s", s.zfsPool, sub, name))
@@ -1004,12 +1020,12 @@ func (s *storageZfs) zfsSnapshotRestore(path string, name string) error {
 }
 
 func (s *storageZfs) zfsSnapshotRename(path string, oldName string, newName string) error {
-	output, err := shared.RunCommand(
+	output, err := exec.Command(
 		"zfs",
 		"rename",
 		"-r",
 		fmt.Sprintf("%s/%s@%s", s.zfsPool, path, oldName),
-		fmt.Sprintf("%s/%s@%s", s.zfsPool, path, newName))
+		fmt.Sprintf("%s/%s@%s", s.zfsPool, path, newName)).CombinedOutput()
 	if err != nil {
 		s.log.Error("zfs snapshot rename failed", log.Ctx{"output": string(output)})
 		return fmt.Errorf("Failed to rename ZFS snapshot: %s", output)
@@ -1019,7 +1035,7 @@ func (s *storageZfs) zfsSnapshotRename(path string, oldName string, newName stri
 }
 
 func (s *storageZfs) zfsMount(path string) error {
-	output, err := shared.TryRunCommand(
+	output, err := tryExec(
 		"zfs",
 		"mount",
 		fmt.Sprintf("%s/%s", s.zfsPool, path))
@@ -1032,7 +1048,7 @@ func (s *storageZfs) zfsMount(path string) error {
 }
 
 func (s *storageZfs) zfsUnmount(path string) error {
-	output, err := shared.TryRunCommand(
+	output, err := tryExec(
 		"zfs",
 		"unmount",
 		fmt.Sprintf("%s/%s", s.zfsPool, path))
@@ -1051,13 +1067,13 @@ func (s *storageZfs) zfsListSubvolumes(path string) ([]string, error) {
 		fullPath = fmt.Sprintf("%s/%s", s.zfsPool, path)
 	}
 
-	output, err := shared.RunCommand(
+	output, err := exec.Command(
 		"zfs",
 		"list",
 		"-t", "filesystem",
 		"-o", "name",
 		"-H",
-		"-r", fullPath)
+		"-r", fullPath).CombinedOutput()
 	if err != nil {
 		s.log.Error("zfs list failed", log.Ctx{"output": string(output)})
 		return []string{}, fmt.Errorf("Failed to list ZFS filesystems: %s", output)
@@ -1086,7 +1102,7 @@ func (s *storageZfs) zfsListSnapshots(path string) ([]string, error) {
 		fullPath = fmt.Sprintf("%s/%s", s.zfsPool, path)
 	}
 
-	output, err := shared.RunCommand(
+	output, err := exec.Command(
 		"zfs",
 		"list",
 		"-t", "snapshot",
@@ -1094,7 +1110,7 @@ func (s *storageZfs) zfsListSnapshots(path string) ([]string, error) {
 		"-H",
 		"-d", "1",
 		"-s", "creation",
-		"-r", fullPath)
+		"-r", fullPath).CombinedOutput()
 	if err != nil {
 		s.log.Error("zfs list failed", log.Ctx{"output": string(output)})
 		return []string{}, fmt.Errorf("Failed to list ZFS snapshots: %s", output)
@@ -1186,17 +1202,17 @@ func storageZFSValidatePoolName(d *Daemon, key string, value string) error {
 		if err != nil {
 			return fmt.Errorf("Invalid ZFS pool: %v", err)
 		}
+	}
 
-		// Confirm that the new pool is empty
-		s.zfsPool = value
-		subvols, err := s.zfsListSubvolumes("")
-		if err != nil {
-			return err
-		}
+	// Confirm that the new pool is empty
+	s.zfsPool = value
+	subvols, err := s.zfsListSubvolumes("")
+	if err != nil {
+		return err
+	}
 
-		if len(subvols) > 0 {
-			return fmt.Errorf("Provided ZFS pool (or dataset) isn't empty")
-		}
+	if len(subvols) > 0 {
+		return fmt.Errorf("Provided ZFS pool (or dataset) isn't empty")
 	}
 
 	// Confirm the old pool isn't in use anymore
@@ -1257,12 +1273,12 @@ func (s *zfsMigrationSourceDriver) send(conn *websocket.Conn, zfsName string, zf
 
 	output, err := ioutil.ReadAll(stderr)
 	if err != nil {
-		logger.Error("problem reading zfs send stderr", log.Ctx{"err": err})
+		shared.LogError("problem reading zfs send stderr", log.Ctx{"err": err})
 	}
 
 	err = cmd.Wait()
 	if err != nil {
-		logger.Error("problem with zfs send", log.Ctx{"output": string(output)})
+		shared.LogError("problem with zfs send", log.Ctx{"output": string(output)})
 	}
 
 	return err
@@ -1368,7 +1384,7 @@ func (s *storageZfs) MigrationSource(ct container) (MigrationStorageSourceDriver
 		}
 
 		lxdName := fmt.Sprintf("%s%s%s", ct.Name(), shared.SnapshotDelimiter, snap[len("snapshot-"):])
-		snapshot, err := containerLoadByName(s.s, s.storage, lxdName)
+		snapshot, err := containerLoadByName(s.d, lxdName)
 		if err != nil {
 			return nil, err
 		}
@@ -1380,7 +1396,7 @@ func (s *storageZfs) MigrationSource(ct container) (MigrationStorageSourceDriver
 	return &driver, nil
 }
 
-func (s *storageZfs) MigrationSink(live bool, container container, snapshots []*Snapshot, conn *websocket.Conn, srcIdmap *idmap.IdmapSet) error {
+func (s *storageZfs) MigrationSink(live bool, container container, snapshots []*Snapshot, conn *websocket.Conn, srcIdmap *shared.IdmapSet) error {
 	zfsRecv := func(zfsName string) error {
 		zfsFsName := fmt.Sprintf("%s/%s", s.zfsPool, zfsName)
 		args := []string{"receive", "-F", "-u", zfsFsName}
@@ -1404,12 +1420,12 @@ func (s *storageZfs) MigrationSink(live bool, container container, snapshots []*
 
 		output, err := ioutil.ReadAll(stderr)
 		if err != nil {
-			logger.Debug("problem reading zfs recv stderr %s", log.Ctx{"err": err})
+			shared.LogDebug("problem reading zfs recv stderr %s", log.Ctx{"err": err})
 		}
 
 		err = cmd.Wait()
 		if err != nil {
-			logger.Error("problem with zfs recv", log.Ctx{"output": string(output)})
+			shared.LogError("problem with zfs recv", log.Ctx{"output": string(output)})
 		}
 		return err
 	}
@@ -1428,7 +1444,7 @@ func (s *storageZfs) MigrationSink(live bool, container container, snapshots []*
 
 	for _, snap := range snapshots {
 		args := snapshotProtobufToContainerArgs(container.Name(), snap)
-		_, err := containerCreateEmptySnapshot(container.StateObject(), container.Storage(), args)
+		_, err := containerCreateEmptySnapshot(container.Daemon(), args)
 		if err != nil {
 			return err
 		}
@@ -1453,7 +1469,7 @@ func (s *storageZfs) MigrationSink(live bool, container container, snapshots []*
 		/* clean up our migration-send snapshots that we got from recv. */
 		zfsSnapshots, err := s.zfsListSnapshots(fmt.Sprintf("containers/%s", container.Name()))
 		if err != nil {
-			logger.Error("failed listing snapshots post migration", log.Ctx{"err": err})
+			shared.LogError("failed listing snapshots post migration", log.Ctx{"err": err})
 			return
 		}
 
